/*

  Graphic user's interface library
  Folder widget

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_FOLDERWIDGET_H
#define FLYFM_FOLDERWIDGET_H

#include <QAbstractItemView>
#include <QElapsedTimer>

#include <flyfm/model.h>
#include <flyfm/imanageiconsize.h>

class QGridLayout;

namespace FlyFM
{
class FOpId;
class Navigation;
class FileActionsProvider;
class IconSizeActionsProvider;
class Job;
class ItemViewTouchGesturesManager;

class FolderWidget : public QWidget, public IManageIconSize
{
    Q_OBJECT
public:
    explicit FolderWidget(QWidget *parent = 0);
    explicit FolderWidget(ModelFilter::EntryFilters, QWidget *parent = 0);
    virtual ~FolderWidget();

    Model *model();
    const Model *model() const;

    void refresh();

    Navigation* navigation();

    Entry currentEntry();
    void setCurrent(const Entry& entry);

    QGridLayout* layout();

    void setHideOpenActions(bool enable);
    void setHideFindAction(bool enable);
    void setHideSendActions(bool enable);
    void setHideViewChooseActions(bool enable);

    static QByteArray defaultState(bool tabletMode);
    QByteArray saveState() const;
    void restoreState(const QByteArray& state);

    IconSize iconSize() const Q_DECL_OVERRIDE;

    void setSelectionMode(QAbstractItemView::SelectionMode mode);

    EntryList selectedEntries() const;

    void setFindMode(bool enable);
    bool isInFindMode() const
    {
        return m_findMode;
    }

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool enable);

    bool showDotDot();
    void setShowDotDot(bool on);

    void setActive(bool active);
    bool isActive() const;

    void setKineticScrollingEnabled(bool on);
    void setPinchZoomEnabled(bool on);

    void setShowLinkEmblem(bool on = true);
    bool showLinkEmblem() const;

    void setShowPropertyEmblems(bool on = true);
    bool showPropertyEmblems() const;

    void setShowSelectionControl(bool on);
    bool showSelectionControl() const;

    void setFileActionsUpdatesEnabled(bool enabled);

    FileActionsProvider* fileActionsProvider();

    QAbstractItemView* itemView() const;

public slots:
    void setIconSize(IconSize size) Q_DECL_OVERRIDE;

    void setEntry(const Entry &);

    void applyTableView();
    void applyIconView();
    void applyListView();

    void highlightEntry(const Entry &);
    void selectEntries(const EntryList& entries);
    void sort(int, Qt::SortOrder);

signals:
    void viewModeChanged();
    void entriesSelected(const EntryList &);
    void openActionRequested(const EntryList&);
    void iconSizeChanged(int);
    void iconSizeChanged(IconSize);
    void currentChanged(const Entry&);
    void activated();
    void itemViewChanged(QAbstractItemView*);

protected slots:
    void updatingStarted();
    void updatingFinished();

    void slotCurrentChanged(const Entry &);
    void dropPopupContextMenu(const QPoint &, const EntryList &, const Entry &);
    void customPopupContextMenu(const QPoint &, const QModelIndexList &);
    void showTableColumnsMenu(const QPoint &);
    void openAction(const QModelIndexList &indexes);
    void activatedAction(const QModelIndex&);
    void upAction();

    void increaseIconSizeAction();
    void decreaseIconSizeAction();
    void selectionChanged();
    void updateModelPresentation();

    void updateShortcutFileAction();
    void updateFileActions();

    void updateSmbEntryJobFinished(Job* job);
    void updateNetworkPlaceJobFinished(Job *job);

protected:
    bool eventFilter(QObject*, QEvent*) Q_DECL_OVERRIDE;
    void changeEvent(QEvent* event) Q_DECL_OVERRIDE;

    void changeViewTo(Model::Mode);
    void setView(QAbstractItemView *view);

    QList<QPair<QString, int> > availablePropertyNames() const;
    QList<QPair<QString, int> > availableSortableProperties() const;

    bool isPropertyHidden(int propertyIdx) const;
    void setPropertyHidden(int propertyIdx, bool hidden);
    int sortPropertyIndex() const;
    Qt::SortOrder sortPropertyOrder() const;

    QMap<Model::Mode, IconSize> m_iconSizeMap;

    mutable QByteArray m_tableHeaderState;
    mutable QList<int> m_tableHiddenSections;

    QList<int> m_hiddenSections;

    Navigation *m_navigation;
    Model *m_model;
    QAbstractItemView *m_view;

    bool hideOpenActions;
    bool hideSendActions;
    bool hideFindAction;
    bool hideViewChooseActions;

    QAbstractItemView::SelectionMode m_selectionMode;

    bool m_findMode;

    QGridLayout* m_layout;

    bool m_isActive;
    bool m_activateItemOnSingleClick;
    ItemViewTouchGesturesManager* m_touchGesturesManager;

    IconSizeActionsProvider* m_iconSizeActionsProvider;
    FileActionsProvider* m_fileActionsProvider;

    bool m_fileActionsUpdatesEnabled;

    QElapsedTimer m_singleClickElapsedTimer;

    friend class ColumnMenu;
    friend class SortMenu;
    friend class PopupMenu;

private:
    void highlightFOpsResult(const Entry &, const FOpId &);
    void init(ModelFilter::EntryFilters entryFilters);

    void localSaveTableHeaderState() const;
    void localRestoreTableHeaderState();

    void setActivateItemOnSingleClick(bool on);

    void setHiddenProperties(const QStringList& props);
    void applyHiddenProperties();

    bool m_showSelectionControl;
};

}

#endif // FLYFM_FOLDERWIDGET_H
