/*

  Graphic user's interface library
  File system model

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_MODEL_H
#define FLYFM_MODEL_H

#include <QAbstractItemModel>

#include <flyfm/modelfilter.h>

namespace FlyFM
{

class Entry;
class EntryList;

class ModelPrivate;

class Model : public QAbstractItemModel
{
    Q_OBJECT
    friend class ModelItem;
public:
    enum Mode { UnspecMode = 0, DesktopMode = 1, IconMode = 2, TreeMode = 4, TableMode = 8, ListMode = 16, PlacesMode = 32 };
    enum SortOrder { UnspecOrder = 0, AscendingOrder = 1, DescendingOrder = 2 };

    explicit Model(Mode mode = DesktopMode, ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries, QObject *parent = 0);
    ~Model();

    void refresh();
    void updatePresentation();

    void rebuild();
    void rebuild(const Entry &rootEntry);
    void rebuildWithMode(Mode mode);
    void rebuildWithEntryFilters(ModelFilter::EntryFilters entryFilters);
    void rebuildWithPropertySortIndex(int propertySortIndex, SortOrder sortOrder = AscendingOrder);

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool enable);

    bool showDotDot();
    void setShowDotDot(bool on);

    ModelFilter& filter();
    const ModelFilter& filter() const;
    void flushFilter();

    Entry actualRootEntry();

    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;

    virtual QModelIndex parent(const QModelIndex &index) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
    virtual bool hasChildren(const QModelIndex &parent = QModelIndex()) const;
    virtual Qt::ItemFlags flags(const QModelIndex &index) const;
    virtual Qt::DropActions supportedDropActions() const;
    virtual bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent);
    virtual QMimeData *mimeData(const QModelIndexList &indexes) const;
    virtual QStringList mimeTypes() const;
    virtual void sort(int column, Qt::SortOrder order);

    virtual QList<QModelIndex> pathIndexList(const Entry &entry);
    virtual QModelIndex indexOfEntry(const Entry &entry);

    // void setPropertyEnabled( const QString &, bool );
    void setNameFilters(const QStringList&);
    void setNameFiltersForFilesOnly(bool on = true);

    Mode mode() const;
    SortOrder sortOrder() const;
    int propertySortIndex() const;
    QList<QPair<QString, int> > sortableProperties() const;

    QStringList propertyNames() const;
    QStringList internalPropertyNames() const;

    ModelFilter::EntryFilters entryFilters() const;
    QStringList nameFilters() const;

    Entry entryByIndex(const QModelIndex &index) const;
    EntryList indexesToEntries(const QModelIndexList &) const;

    void setShowLinkEmblem(bool on = true);
    bool showLinkEmblem() const;

    void setShowPropertyEmblems(bool on = true);
    bool showPropertyEmblems() const;

    static bool isHiddenChild(const Entry &);

signals:
    void updatingStarted();
    void updatingFinished();
    void updated(const QModelIndex &);

protected:
    QScopedPointer<ModelPrivate> d_ptr;
    Model(ModelPrivate& dd, Mode mode = DesktopMode, ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries, QObject *parent = 0);

    void beginInsertRows(const QModelIndex &parent, int first, int last);
    void endInsertRows();

private:
    Q_DECLARE_PRIVATE(Model)

    Q_PRIVATE_SLOT(d_func(), void _q_entryCreated(const Entry&))
    Q_PRIVATE_SLOT(d_func(), void _q_entryDeleted(const Entry&))
    Q_PRIVATE_SLOT(d_func(), void _q_entryChanged(const Entry&))
    Q_PRIVATE_SLOT(d_func(), void _q_entryAttributesChanged(const Entry&))
    Q_PRIVATE_SLOT(d_func(), void _q_entryReset(const Entry&))

    Q_PRIVATE_SLOT(d_func(), void _q_dataHoldersUpdated(const EntryPropertyRequestList&))
};

}

QDataStream & operator<< (QDataStream & out, const FlyFM::Model::Mode& mode);
QDataStream & operator>> (QDataStream & in, FlyFM::Model::Mode& mode);

#endif // FLYFM_MODEL_H
