/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_VFS_H
#define FLYFM_VFS_H

#include <QObject>

#include <flyfm/storageplace.h>
#include <flyfm/networkplace.h>

namespace FlyFM
{

class Entry;
class EntryList;
class VFSPlaceBase;
class EntryWatcher;
class FavouritePlace;
class StorageDeviceInfo;

class SmbBrowserProxy;
class NetworkMountsManagerProxy;
class NetworkPlacesManagerProxy;
class FuseMountsManagerProxy;

class VFSStorageMountJob;
class VFSStorageUnmountJob;
class VFSStorageEjectJob;
class VFSSettings;

class VFSPrivate;

class VFS : public QObject
{
    Q_OBJECT

    friend class Entry;
    friend class VFSPlaceBase;

public:
    static void initialize();

    static VFS *ptr();
    ~VFS();

    VFSSettings* settings();
    const VFSSettings* settings() const;

    Entry entry(const QString &path) const;
    // QString schemeForEntry(const Entry &entry);

    bool hasParent(const Entry &entry) const;
    bool hasChilds(const Entry &entry) const;
    bool isEntryExists(const Entry &entry) const;
    Entry parent(const Entry& entry) const;
    Entry parent(const Entry& entry, const Entry& root) const;

    enum EntryChildFilter
    {
        NoEntryChildFilter     = 0x00000000,
        SkipSymLinks           = 0x00000001,
        WithArchiveMountPoints = 0x00000002
    };

    Q_DECLARE_FLAGS(EntryChildFilters, EntryChildFilter)

    EntryList childs(const Entry &parent, EntryChildFilters entryChildFilters = NoEntryChildFilter) const;
    Entry child(const Entry& parent, const QString& fileName) const;
    bool containsDirsOrPlaces(const Entry &entry) const;
    Entry symLinkTargetEntry(const Entry& symLinkEntry) const;

    void stopGettingChilds();

    QString localPath(const QString &url) const;
    QString localPath(const Entry &entry) const;
    QList<QUrl> urlsFromEntries(const EntryList &entries) const;
    EntryList entriesFromUrls(const QList<QUrl> &urls) const;

    VFSStorageMountJob* mount(const Entry &entry);
    VFSStorageUnmountJob *unmount(const Entry &entry);
    VFSStorageEjectJob *eject(const Entry &entry);

    StoragePlace storagePlace(const Entry &entry) const;
    StoragePlaceList storagePlacesByType(StoragePlace::Type type) const;
    StoragePlace freshStoragePlace(const StoragePlace &) const;

    StorageDeviceInfo storageDeviceInfo(const StoragePlace &storagePlace) const;

    FavouritePlace favouritePlace(const Entry& entry) const;

    NetworkPlace networkPlace(const Entry& entry) const;

    EntryList pathEntryList(const Entry &entry) const;
    EntryList pathEntryList(const Entry &entry, const Entry& root) const;
    QStringList fileSystemEntryPathList(const Entry& root) const;

    EntryWatcher *entryWatcher();

    VFSPlaceBase *placeBase();
    const VFSPlaceBase *placeBase() const;

    void notifyAllWatchers(const EntryList &entries);
    void notifyWatchers(const EntryList &entries);
    void notifyWatchers(const Entry &entry);

    /** Removes all multiple directory separators "/" and resolves any "."s or ".."s found in the path */
    QString normalizePath(const QString &path) const;

    bool isSupportedFileArchive(const Entry &) const;

    bool isEntryInNetwork(const Entry& entry) const;
    bool hasNetworkRelativeEntries(const EntryList& entries) const;

    bool hasStoragePlaceRelativeEntries(const EntryList &entries) const;
    bool isStoragePlaceRelativeEntry(const Entry &entry) const;
    bool isPlaceRelativeEntry(const Entry &entry) const;

    Entry rootEntry() const;
    Entry favouritesEntry() const;
    Entry computerEntry() const;
    Entry placesEntry() const;
    Entry networkEntry() const;
    Entry sambaNetworkEntry() const;
    Entry ftpNetworkEntry() const;
    Entry sftpNetworkEntry() const;
    Entry desktopEntry() const;
    Entry trashEntry() const;
    Entry homeEntry() const;
    Entry fileSystemEntry() const;
    Entry documentsEntry() const;
    Entry storageDevicesEntry() const;

    Entry placeRelatedEntry(const Entry& entry) const;

    void directNotifyWatchers(const EntryList&, bool anyWay = false);

    //Entry mountPointFor(const Entry &);
    //ProcessInfoList unmountFuseEntry(const Entry& entry);

    bool hasFavouritePlace(const FavouritePlace& place) const;
    void addFavouritePlace(const FavouritePlace& place);
    void removeFavouritePlace(const FavouritePlace& place);
    QString favouritePlacePath(const FavouritePlace& place) const;

    QStringList exploreParentPlacesHistory(const QString &path) const;
    QStringList exploreParentPlacesHistory(const QString &path, const Entry& root) const;

    SmbBrowserProxy* smbBrowser();
    NetworkMountsManagerProxy* networkMountsManager();
    NetworkPlacesManagerProxy* networkPlacesManager();
    FuseMountsManagerProxy* fuseMountsManager();

    QString smbWorkgroup(const Entry& entry) const;
    QString smbHost(const Entry& entry) const;

    Entry smbShareEntry(const QString& share, const QString& host, const QString& workgroup) const;

    Entry createEntryForUrl(const QString& urlStr, const QString& localPath) const;
    QString getUrlForEntry(const Entry& entry) const;

signals:
    void storagePlaceMountStateChanged(const StoragePlace &storagePlace);
    void storagePlaceMountabilityChanged(const StoragePlace &storagePlace);
    void storagePlaceRemoved(const StoragePlace &storagePlace);
    void smbShareUpdated(const Entry& entry);
    void networkPlaceUpdated(const Entry& entry);
    void networkPlaceAdded(const NetworkPlace& networkPlace, const Entry& entry);
    void fuseMountAdded(const QString& url, const Entry& entry);

    void longTimeOperationStarted();
    void longTimeOperationFinished();

private:
    VFS();

    QScopedPointer<VFSPrivate> d_ptr;
    Q_DECLARE_PRIVATE(VFS)

    Q_PRIVATE_SLOT(d_func(), void _q_serviceNotifyWatchers(const FlyFM::EntryList&))
    Q_PRIVATE_SLOT(d_func(), void _q_serviceStoragePlaceUpdated(const FlyFM::StoragePlace &, const FlyFM::StoragePlace &))
    Q_PRIVATE_SLOT(d_func(), void _q_serviceStoragePlaceAdded(const FlyFM::StoragePlace &))
    Q_PRIVATE_SLOT(d_func(), void _q_serviceStoragePlaceRemoved(const FlyFM::StoragePlace &))

    Q_PRIVATE_SLOT(d_func(), void _q_smbWorkgroupsChanged(QStringList, QStringList))
    Q_PRIVATE_SLOT(d_func(), void _q_smbWorkgroupUpdated(QString, QStringList, QStringList))
    Q_PRIVATE_SLOT(d_func(), void _q_smbHostUpdated(QString, QString, StringMap, StringMap))
    Q_PRIVATE_SLOT(d_func(), void _q_smbShareUpdated(QString, QString, QString, QString))
    Q_PRIVATE_SLOT(d_func(), void _q_fuseMountAdded(QString, FlyFM::Entry))

    Q_PRIVATE_SLOT(d_func(), void _q_createSambaWorkgroups(const QString&))
    Q_PRIVATE_SLOT(d_func(), void _q_createSambaHosts(const QString&))
    Q_PRIVATE_SLOT(d_func(), void _q_createSambaShares(const QString&))
    Q_PRIVATE_SLOT(d_func(), void _q_createStoragePlaces(const QString&))

    Q_PRIVATE_SLOT(d_func(), void _q_favouritePlaceAdded(const FlyFM::FavouritePlace& place))
    Q_PRIVATE_SLOT(d_func(), void _q_favouritePlaceRemoved(const FlyFM::FavouritePlace& place))

    Q_PRIVATE_SLOT(d_func(), void _q_networkPlaceAdded(const FlyFM::NetworkPlace& place))
    Q_PRIVATE_SLOT(d_func(), void _q_networkPlaceRemoved(const FlyFM::NetworkPlace& place))
    Q_PRIVATE_SLOT(d_func(), void _q_networkPlaceUpdated(const FlyFM::NetworkPlace& place))
};

}

#endif
